/***********************************************************/
/* TinyText.exf  by Aaron J. Miller                        */
/* millera@stanford.edu                                    */
/* Copyright (c) 1999                                      */
/* Released to public domain for any non-commercial use.   */
/***********************************************************/

#include "psdos.h"
#include "lcdbios.h"
#include "wbios.h"
#include "rupsys.h"

#define MAXSIZE 16384
#define MAXLN MAXSIZE/10
char font[128*8+4];
char dbuff[102 * 8];  /* double buffer for fast screen updates */
char mod8[]={0,6,4,2,0,6,4,2,0,6};  /* lookup table for shifts */
unsigned int offset[]={0,0,102,204,306,306,408,510,612,612};   /*lookup table to avoid mulitplications */
char filebuff[MAXSIZE]; /* buffer to hold the file */
unsigned int scroll[MAXLN+1]; /* nonelegant way to store page lengths */

int dbuffstr(char *str)
{

	char y=0,y2,i,x,wrapped;

	char *tmp,*strtmp;

  int idx;

  char *tempint;

  unsigned int lkup,retval;


  strtmp=str;  /* keep track of original string pointer */

	memset(dbuff,0x00,102*8); /* clear the screen memory */


  do

  {

    x=0;

    wrapped=1;

    do  /* max of 25 characters wide */

    {

      if (!*str) {    /* end of string? */

        goto out;     /* I like gotos... so bite me. */

      }

      preproc:
        switch (*str) {
          case 0x0D:
            if (*(str+1) == 0x0A) str++;
          case 0x0A:
            str++;
            if (!(wrapped && (x==0))) {
              x=0;
              wrapped=0;
              if (++y >9) goto out;
            }
            goto preproc;
          case '\t':    
            str++;
            if (x==0) x=12;
            goto preproc;
          default:
            break;
        }

      y2=mod8[y];
      tmp=dbuff+ offset[y] +x;
      idx= (*str)<<2;

      tempint=font+idx;

      /* the following used to be a for loop that I unrolled */

        lkup= *(tempint++)<<y2;
        *tmp |= (char)lkup;
        tmp+=102;
        *tmp |= (char)((lkup&0xFF00)>>8);
        tmp-=101;

        lkup= *(tempint++)<<y2;
        *tmp |= (char)lkup;
        tmp+=102;
        *tmp |= (char)((lkup&0xFF00)>>8);
        tmp-=101;

        lkup= *(tempint++)<<y2;
        *tmp |= (char)lkup;
        tmp+=102;
        *tmp |= (char)((lkup&0xFF00)>>8);
        tmp-=101;

        lkup= *(tempint)<<y2;
        *tmp |= (char)lkup;
        tmp+=102;
        *tmp |= (char)((lkup&0xFF00)>>8);
        tmp-=101;

      str++;
      x+=4;
    } while (x<100);  /* 25 columns */
    y++;
  } while (y<=9);    /* max of 10 lines per screen */

  out:

  gv_aput(0,0,102,64,dbuff,0);
  retval = (str-strtmp);


  return retval;

}


int main()

{
	int bt,ret,err=0;
	int line;
	int i,f;
  long maxlen;
  unsigned int maxline=MAXLN;
  char *buffp;
  char *argv;
  char fontname[50];

  argv=getArg();
	screen(1);
	gv_place(0,0);
	cls(4);
  f=dos_fopen(argv,0);
  if (f<0) {
    gv_kput(0,30,"can't open file",2,0,0);
    err=1;
  }
  else {
    gv_kput(0,30,"wait...",2,0,0);
    maxlen=dos_fseek(f,2,0);   /* seek to eof */
    dos_fseek(f, 0, 0);
    if (maxlen<MAXSIZE) {
      dos_fread(f,filebuff,maxlen);
    }
    else {
      gv_kput(0,10,"file too big.",2,0,0);
      err=1;
    }
    dos_fclose(f);
    buffp=filebuff;
    i=0;
    if (filebuff[0]==0)  {
      gv_kput(0,30,"Not a text file.",2,0,0);
      err=1;
    } else
    {
      if ((buffp[0]=='<') && (buffp[1]=='T')) {
        buffp+=6;                 // just skip the TFNT= part right now
        while (*buffp!='>') fontname[i++]=*(buffp++);  //copy the filename
        fontname[i]=0;
        buffp++;
      } else strcpy(fontname,"\\system\\mmp\\tinyfont.mmp");
    }
  }
  if ((!err) && (mpread(font,fontname))) {
    gv_kput(0, 22,fontname,2,0,0);
    gv_kput(0, 30, "font not found", 2, 0, 0);
    err=1;
  }
  else {
    /* show the first page */
    scroll[0]=0;
    line=1;
    scroll[line]=dbuffstr(buffp);
    if (!*(buffp+scroll[line])) {
      scroll[line]=0;
      maxline=line;
    }
    /* now wait for key press */
    while (1) {

      bt = Keyin((Bkey_up|Bkey_dw), -1);
      ret = EventCall(bt);
      if (ret==-1) dos_exit(0); //return;   // application termination request
      else if (ret==1) {
        ret = EventExec();   // application timeout?
        if (ret==-1) dos_exit(0); //return; // application termination request
      }
      if (bt & Bkey_D) dos_exit(0);
      else if (bt & Bkey_up) {
        if (line>1) {
          buffp-=scroll[--line];
          dbuffstr(buffp);
        }
      }
      else
      if( bt & Bkey_dw) {
        if (line < maxline) {
          buffp+=scroll[line++];
          scroll[line]=dbuffstr(buffp);
          if (!*(buffp+scroll[line])) {
            scroll[line]=0;
            maxline=line;
          }
        }
      }
      else if (bt) {
        dbuffstr(buffp);  //redraw
      }
    }
  }
  if (err) while(!bi_getbtn());
  cls(4);
}
